<?php
// $Header: /home/hiro/projects/tv/RCS/6ga-common.inc,v 1.34 2011/11/02 05:27:52 hiro Exp $
//
// 6ga-common.inc
//
//	Include file for common definitions and functions
//
//	Copyright (c) 2005 Hiro Sugawara
//	Distributed under terms of GPL 2.0
//
if (!defined('PHP_VERSION_ID')) {
	$version = explode('.', PHP_VERSION);

	define('PHP_VERSION_ID',
		($version[0] * 10000 + $version[1] * 100 + $version[2]));
}
if (PHP_VERSION_ID >= 50100) {
	date_default_timezone_set('Asia/Tokyo');
}

openlog('6ga', 0, LOG_LOCAL0);

$isdvr2		= @$_COOKIE['dvr2'];
$thistab	= @$_GET['tab'];
if (!@$thistab) {
	$thistab	= @$_COOKIE['tab'];
}
setcookie('tab', $thistab);
$thisfile	= @$_GET['file'];
$choice		= @$_GET['choice'];
$action		= @$_GET['action'];
$bgcolor	= @$_GET['bgcolor'];
$program	= @$_GET['program'];

//
// Read in configuration file
//
$DOCROOTDIR	= dirname($_SERVER['SCRIPT_FILENAME']);
$CONFIGROOTDIR	= "$DOCROOTDIR/config";
$CONFIGFILENAME	= "$CONFIGROOTDIR/tv.conf";
putenv("DOCROOTDIR=$DOCROOTDIR");
putenv("CONFIGROOTDIR=$CONFIGROOTDIR");
putenv("CONFIGFILENAME=$CONFIGFILENAME");

$config	= fopen($CONFIGFILENAME, 'r');
if ($config) {
	while (!feof($config)) {
		$line = fgets($config);
		eval($line);
	}
	fclose($config);
}
putenv("DOCBASEURI=$DOCBASEURI");
$RCMDPATH	= '/home/rec/bin';
$SUDO	= 'sudo -u rec';
putenv('PATH=$RCMDPATH:' . getenv('PATH'));

//
// Common constant arrays
//
$TABS		= array('scheduled', 'recorded', 'regular');
$TABCOLOR	= array('scheduled'	=> '#ffcfcf',
					'recorded'	=> '#b0f0b0',
					'regular'	=> '#ffff80');

$FNAMEPATTERN
		= array('scheduled'	=> '\\d{12}-\\d{3}-\\d{4}-[[:xdigit:]]{8}',
				'recorded'	=> '\\d{12}-\\d{3}-\\d{4}-[[:xdigit:]]{8}',
				'regular'	=> '\\d{7}-\\d{4}-\\d{3}-\\d{4}-[[:xdigit:]]{8}');
$FNAMESORTER	= array('scheduled'	=> 'sort',
						'recorded'	=> 'rsort',
						'regular'	=> 'regsorter');

// Digital station map
$station = array();
$channel = array();
include("$CONFIGROOTDIR/channels/$dCHANNELMAPphp");

//
// Common logger
//
function logit($str)
{
	syslog(LOG_INFO,
			sprintf('%s: %s', basename($_SERVER['SCRIPT_FILENAME']), $str));
}

//
// Special sorter for regular schedules based on DOW map
//
function regcmp($a, $b) {
	if (substr($a, 0, 7) == substr($b, 0, 7))
		return strcmp(substr($a, 7), substr($b, 7));
	return strcmp(substr($b, 0, 7), substr($a, 0, 7));
}

function regsorter(&$fbnames) {
	usort($fbnames, 'regcmp');
}

//
// Open a list of IEPG files
//
//	$filter:	callback for fitering files
//	$farg:		argument to filtering callback
//	$suffix:	file suffix
//
function openList($tab, $filter = '', $farg = '', $suffix = 'iepg',
					$sorter = '') {
	global $VIDEOROOTDIR;
	global $FNAMEPATTERN, $FNAMESORTER;

	$iepgFNPattern	= sprintf('/^%s\.%s$/', $FNAMEPATTERN[$tab], $suffix);

	$dirpath	= "$VIDEOROOTDIR/$tab";
	$list	= array('dirpath'	=> $dirpath,
					'tab'		=> $tab);
	if (($dhandle	= opendir($dirpath)) == FALSE)
		return FALSE;

	$list['fbasenames']	= array();
	while (FALSE !== ($fname = @readdir($dhandle))) {
		if (preg_match($iepgFNPattern, $fname)) {
			$fbasename	= basename($fname, ".$suffix");
			if ($filter && !$filter($fbasename, $farg))
				continue;
	
			$list['fbasenames'][]	= $fbasename;
		}
	}

	closedir($dhandle);

	if (!$sorter)
		$sorter	= $FNAMESORTER[$tab];
	if ($sorter)
		$sorter($list['fbasenames']);
	return $list;
}

function closeList($list) {
}

//
// Read next iEPG file in the list
//
function readList(&$list, $fnbase = 0) {
	global $FNAMEPATTERN;

	$iepgFNPattern	= sprintf('/^%s\.iepg$/', $FNAMEPATTERN[$list['tab']]);
	$aviFNPattern	= sprintf('/^%s\.iepg$/', $FNAMEPATTERN[$list['tab']]);

	if ($fnbase) {
		foreach (array_values($list['fbasenames']) as $fnb) {
			if ($fnbase == $fnb)
				break;
		}
		if ($fnbase != $fnb)
			return FALSE;
		$list['fbasename']	= $fnb;
	} elseif (!($list['fbasename'] = array_shift($list['fbasenames']))) {
		return FALSE;
	}

	$list['filepath']	= $list['dirpath'] . '/' . $list['fbasename'] . '.iepg';
	if (($iepgfh	= @fopen($list['filepath'], 'r')) == FALSE)
		return FALSE;

	$list['iepg']	= array();	// Clear all fields
	$inHeader	= TRUE;
	while (!feof($iepgfh)) {
		$line = trim(fgets($iepgfh));
		if ($inHeader) {
			if ($line == '') {
				$inHeader	= FALSE;
			} else {
				$header	= preg_split('/:\s+/', $line, 2);
				preg_replace('/\\\'/', '\'', @$header[1]);
				$list['iepg'][$header[0]] = @$header[1];
			}
		} else {
			$list['iepg-body'] .= "$line\n";
		}
	}
	fclose($iepgfh);
	
	return TRUE;
}

//
// Get day of the week from an iEPG list
//
function getDOW($iepg) {
	$start	= preg_split('/:/', $iepg['start']);
	return date('D', mktime($start[0], $start[1], 0,
				$iepg['month'], $iepg['date'], $iepg['year']));
}

//
// Get start time of recording
//
function getStartTime($iepg) {
	$start	= preg_split('/:/', $iepg['start']);
	// php mktime is not POSIX compliant
	return mktime($start[0], $start[1], 0,
				$iepg['month'], $iepg['date'], $iepg['year']);
}

//
// Trivial library to save iEPG data from arguments to an array
//
function saveiepg() {
	global $_GET, $iepg, $station, $channel;
	$cset = 'shift_jis';
	$iepg	= array();

	foreach (array_keys($_GET) as $key) {

		if (preg_match('/^iepg_/', $key)) {
			$val = $_GET[$key];
			// Escape single quotes (apostrophes) passed in URI format
			$key = substr($key, 5);
			$iepg[$key] = preg_replace("/%27/", "\\\'", $val);
			if (preg_match('/^[Cc]ontent-[Tt]ype$/', $key) &&
				preg_match('/charset=([\w\-]+)/', $val, $matches)) {
				$cset = $matches[1];
			}
			$cset2 = mb_detect_encoding($val, 'utf-8,shift_jis');
			if ($cset2 != 'shift_jis')
				$cset = $cset2;
		}
	}
	unset($key);

	// Convert charset to shift_jis
	if (!preg_match('/^shift[_\-]?jis$/i', $cset)) {
		foreach ($iepg as $key => &$val) {
			$iepg[$key] = iconv($cset, 'shift_jis', $val);
			if (preg_match('/^[Cc]ontent-[Tt]ype$/', $key)) {
				$val =
					preg_replace("/charset=$cset/", 'charset=shift_jis', $val);
			}
		}
		unset($val);
	}

	// Remove [X] icons
	foreach ($iepg as $key => &$val) {
		$val = preg_replace('/\x81\x79..\x81\x7a/', '', $val);

	}
	unset($val);

	// Convert digital to analog channel
	if (preg_match('/D:(\d+ch)/', $iepg['station'], $matches)) {
		$iepg['d-channel'] = 'GR' . $matches[1];
		$iepg['station'] = $station[$matches[1]];
		$iepg['channel'] = $channel[$matches[1]];
	}
}

//
// Assemble file name path
//
function filenamepath($list, $suffix) {
	return sprintf("%s/%s.%s",
				$list['dirpath'], $list['fbasename'], $suffix);
}

//
// For debug use
//
function showGETArgs() {
	global $_GET;
	$first	= 1;

	print("<!-- ");
	foreach (array_keys($_GET) as $key) {
		printf("%c%s=%s", $first ? '?' : '&', $key, $_GET[$key]);
	}
	print(" -->\n");
}

//
// Calculate number of minutes of the program
//
function programTime($iepg, $recording = 0) {
	$start	= $recording ? 'start-recording'	: 'start';
	$end	= $recording ? 'end-recording'		: 'end';
	$shm	= @preg_split('/:/', $iepg[$start]);
	$ehm	= @preg_split('/:/', $iepg[$end]);
	if (!$shm || !$ehm)
		return 0;
	$hour	= $ehm[0] - $shm[0];
	$min	= $ehm[1] - $shm[1];
	if ($min < 0) {
		$min += 60;
		$hour--;
	}
	if ($hour < 0) {
		$hour += 24;
	}
	return $hour * 60 + $min;
}

//
// Print document header
//
function printDocHeader() {
	global $_SERVER;

	header('Content-Type: text/html;charset=shift_jis', TRUE);
	$file	= $_SERVER['PHP_SELF'];
	$uri	= $_SERVER['REQUEST_URI'];
	echo <<<END
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2 Final JP">
<!---------------------- generated by $file ----------------------->
<!-------- $uri ------------->

END;
}

//
// As of php 5.0.4, many (if not all) file manipulation functions fail
// if the file size is no smaller than 2GB. Here are the alternatives.
//

function file_exists_2gb($filename) {
	system("test -f $filename", $rval);
	return ($rval == 0);
}

//
// Get file size in kilo (=1000) bytes
//
function fsizeInKB($fname) {
	$fsize	= 0;
	if (($fh = popen("ls -l --block-size=1000 $fname|awk '{print \$5}'",'r'))) {
		$fsize	= trim(fgets($fh, 13));
		pclose($fh);
	}
	return $fsize;
}

//
// Get file size in bytes
//
function fsizeInB($fname) {
	$fsize	= "";
	if (($fh = popen("ls -l $fname|awk '{print \$5}'",'r'))) {
		$fsize	= trim(fgets($fh, 13));
		pclose($fh);
	}
	return "$fsize";
}

//
// Read out stat file
//
function read_file($fpath) {
	if (!($fh = @fopen($fpath, 'r')))
		return NULL;

	$content = trim(fgets($fh, 50));
	fclose($fh);
	return $content;
}

//include_once('6ga-common2.inc');
?>
