<?php
// $Id: 6ga-list.php,v 1.66 2011/11/06 23:27:34 hiro Exp $
//
// 6ga-list.php
//
//	Copyright (c) 2005 Hiro Sugawara
//	Licensed under terms of GPL 2.0
//
//	Common listing script
//
//	Arguments:
//		tab=xyz		common 6ga tab/folder name
//		file=xyz	file to be listed (default=all);
//
include('6ga-common.inc');

//
// Listing utilities
//
$LISTHEADERCOLOR	= '#8080ff';
$LISTBGCOLOR	= array('#f7f7f7', '#e0e0ff');

$listBGIndex	= 0;
function getListBGColor() {
	global $LISTBGCOLOR, $listBGIndex;

	$listBGIndex++;
	$listBGIndex %= 2;
	return $LISTBGCOLOR[$listBGIndex];
}

//
// Listing parameter definitions
//
$COLWIDTH	= array('STATION'	=> 35,
					'DATETIME'	=> 60,
					'SCHEDULE'	=> 60,
					'TITLE'		=> 0,
					'SIZE'		=> 40,
					'STATUS'	=> 50,
					'CANCEL'	=> 50);
$COLHEAD	= array('STATION'	=> 'Station',
					'DATETIME'	=> 'Date<BR>Time',
					'TITLE'		=> 'Title/Description',
					'SCHEDULE'	=> 'Schedule',
					'SIZE'		=> 'Size',
					'STATUS'	=> 'Status',
					'CANCEL'	=> 'Cancel');
$COLHALIGN	= array('STATION'	=> 'center',
					'DATETIME'	=> 'left',
					'SCHEDULE'	=> 'left',
					'TITLE'		=> 'center',
					'SIZE'		=> 'center',
					'STATUS'	=> 'center',
					'CANCEL'	=> 'center');
$COLALIGN	= array('STATION'	=> 'left',
					'DATETIME'	=> 'left',
					'SCHEDULE'	=> 'left',
					'TITLE'		=> 'left',
					'SIZE'		=> 'right',
					'STATUS'	=> 'center',
					'CANCEL'	=> 'center');
$COLSHOW	= array('STATION'	=> 'showStation',
					'DATETIME'	=> 'showDateTime',
					'SCHEDULE'	=> 'showSchedule',
					'TITLE'		=> 'showTitle',
					'SIZE'		=> 'showSize',
					'STATUS'	=> 'showStatus',
					'CANCEL'	=> 'showCancel');

$COLUMN	= array('scheduled'	=> array('STATION',
									 'DATETIME',
									 'TITLE',
									 'CANCEL'),
				'recorded'	=> array('STATION',
									 'DATETIME',
									 'TITLE',
									 'SIZE',
									 'STATUS'),
				'regular'	=> array('STATION',
									 'SCHEDULE',
									 'TITLE',
									 'CANCEL'));
$NOFCOLS		= array('scheduled'	=> 4,
						'recorded'	=> 5,
						'regular'	=> 4);

//
// Read stat file
//
function readStat($list) {
	if (($sfile	= @fopen(filenamepath($list, 'stat'), 'r')) != NULL) {
		$stat	= trim(@fgets($sfile));
		fclose($sfile);
	}
	if (!@$stat) {
		$stat	= 'unknown';
	}
	return $stat;
}

//
// Read xfrd file
//
function readXfrd($list) {
	$xfrd	= '0';
	if (($xfile	= @fopen(filenamepath($list, 'xfrd'), 'r')) != NULL) {
		$xfrd	= trim(fgets($xfile));
		fclose($xfile);
		if (!$xfrd)
			$xfrd	= 0;
	}
	return $xfrd;
}

//
// Common column heder display function
//
function showListHeader() {
	global $thistab;
	global $LISTHEADERCOLOR;
	global $COLWIDTH, $COLHEAD, $COLHALIGN, $COLUMN;

	printf("<TR HEIGHT=32 BGCOLOR='$LISTHEADERCOLOR'>\n");
	// First, calculate variable column width
	$cw	= 0;
	foreach ($COLUMN[$thistab] as $col) {
		$cw	+= $COLWIDTH[$col];
	}
	$vcw	= 340 - $cw;
	foreach ($COLUMN[$thistab] as $col) {
		printf("<TH WIDTH=%d ALIGN='%s'>%s</TH>",
				$COLWIDTH[$col] ? $COLWIDTH[$col] : $vcw,
				$COLHALIGN[$col], $COLHEAD[$col]);
	}
	printf("</TR>\n");
}

//
// Individual column display functions
//
function showStation($list) {
	// NHK, TBS, TEREBI
	$hkeys	= array("\x82m\x82g\x82j", "\x82s\x82a\x82r", "\x83e\x83\x8C\x83r");
	// TEREBI
	$tkeys	= array("\x83e\x83\x8C\x83r");
	$station	= $list['iepg']['station'];
	$ss	= NULL;

	foreach ($hkeys as $key) {
		if (!strncmp($station, $key, strlen($key))) {
			$ss	= "$key<BR>&nbsp;" . substr($station, strlen($key));
		}
	}
	if (!@$ss) {
		foreach ($tkeys as $key) {
			if (!strcmp(substr($station, -strlen($key)), $key)) {
				$ss	= substr($station, 0, strlen($station) - strlen($key))
						. "<BR>&nbsp;$key";
			}
		}
	}
	if (!@$ss) {
		$ss	= $station;
	}
	printf("<TD>%s</TD>\n", $ss);
}

//
// Show program's recording time
//
function showDateTime($list) {
	$minutes	= programTime($list['iepg']);
	$dow	= @$list['iepg']['every']
				? sprintf("<I>%s</I>", getDOW($list['iepg']))
				: getDOW($list['iepg']);
	printf("<TD><FONT SIZE='-1'>%s/%s&nbsp;[%s]<BR>%s&nbsp;%dh%02dm</FONT></TD>\n",
			$list['iepg']['month'],
			$list['iepg']['date'],
			$dow,
			$list['iepg']['start'],
			$minutes/60, $minutes%60);
}

//
// Show program's schedule time
//
function showSchedule($list) {
	$minutes	= programTime($list['iepg']);
	printf("<TD><FONT SIZE='-1'>Every&nbsp;%s<BR>%s&nbsp;%dh%02dm</FONT></TD>\n",
			$list['iepg']['every'],
			$list['iepg']['start'],
			$minutes/60, $minutes%60);
}


//
// Show program title
//
function showTitle($list) {
	$stat	= readStat($list);
	$subtitle	= @$list['iepg']['program-subtitle'];
	if (!$subtitle)
		$subtitle	= @$list['iepg']['performer'];
	$subtitle = preg_replace('/\x81\x42.*$/', '', $subtitle);
	$ftag1	= '';
	$ftag2	= '';
	if (strlen($subtitle) > 50) {
		$ftag1	= '<FONT SIZE=\'-1\'>';
		$ftag2	= '</FONT>';
	}

	//
	// Check accuracy of Title
	//
	$iopen	= '';
	$iclose	= '';
	if ($orig_title	= @$list['iepg']['program-title-ORIGINAL']) {
		// Rmove bracketed strings
		$orig_title	= preg_replace('/\[[^\]]*\]/', '', $orig_title);
		$title		= preg_replace('/\[[^\]]*\]/', '',
								$list['iepg']['program-title']);
		// Remove NAKAGURO
		$orig_title	= preg_replace('/\x81\x45/', '', $orig_title);
		$title		= preg_replace('/\x81\x45/', '', $title);

		// Remove KAGIKAKKO
		$orig_title	= preg_replace('/\x81u/', '', $orig_title);
		$title		= preg_replace('/\x81u/', '', $title);
		$orig_title	= preg_replace('/\x81v/', '', $orig_title);
		$title		= preg_replace('/\x81v/', '', $title);

		// Compare original and up-to-date titles with tolerance
		$minlen		= min(strlen($title), strlen($orig_title));
		if ($title != $orig_title
			&& strncmp($title, $orig_title, $minlen)
			&& strcmp(substr($title,-$minlen), substr($orig_title,-$minlen))) {
			$iopen	= '<B>? ';
			$iclose	= ' ?</B><BR><I>(' . $orig_title . ')</I>';
			$subtitle	= NULL;
		} else if (@$list['iepg']['iepg-verified'] != 'yes') {
			$iopen	= '<B>? ';
			$iclose	= ' ?</B>';
			if (@$list['iepg']['scheduled'] == 'skipped') {
				$iclose	= ' ?<BR>**TO BE SKIPPED**</B>';
			}
		}
	}
	printf("<TD>%s%s%s%s</TD>\n",
			$iopen,
			$list['iepg']['program-title'],
			$iclose,
			$subtitle ? "<BR>$ftag1$subtitle$ftag2" : '');
}

//
// Show data file size
//
function showSize($list) {
	global $TRANSCODEBITRATE, $AUDIOBITRATE;

	// Try different suffice in priority order
	$suffice	= array('avi', 'mpeg2', 'mpg2', 'mpeg', 'mpg');
	$align		= 'right';
	$percent	= '';

	foreach ($suffice as $suffix) {
		$fname	= filenamepath($list, $suffix);
		// php's internal stat-family function cannot handle
		// files bigger than 2GB...
		if (file_exists_2gb($fname)) {
			$stext = exec("ls -l --si $fname | awk '{print \$5}'");
			break;
		}
	}

	$tsfile = @$list['iepg']['tsfile'];
	$tsdir = @$list['iepg']['tsdir'];
	$avidir = @$list['iepg']['avidir'];
	$stat	= readStat($list);

	$dir = '';
	if ($tsfile && ($tsdir || $avidir)) {
		if (preg_match('/decrypt/i', $stat) || preg_match('/decod/i', $stat)) {
			$dir = $avidir;
		} else if ($stat == 'Recording') {
			$dir = $tsdir;
		}

		if ($dir && file_exists_2gb("$dir/$tsfile")) {
			$stext = exec("ls -l --si $dir/$tsfile | awk '{print \$5}'");
		}
	}

	// dvr2 ffmpeg uses b/s, not kb/s
	$transcodebitrate = @$list['iepg']['vbitrate'];
	if (!$transcodebitrate)
		$transcodebitrate = $TRANSCODEBITRATE;
	$transcodebitrate /= 1000;
	$audiobitrate = $AUDIOBITRATE/1000;

	if (!@$stext) {
		$stext	= '---';
		$align	= 'center';
	} else {
		if ($stat == 'Transcoding') {
			$secs	= programTime($list['iepg'], 1) * 60;
			$fsize	= fsizeInKB($fname);

			$pct	= $fsize/($secs * ($transcodebitrate + $audiobitrate)/8)
								* 100 + 0.5;
			if ($pct > 99) {
				$pct = 99;
			}
			$percent = sprintf("<BR>(%d%%)", $pct);
		} else if ($stat == 'Recording') {
			$secs	= programTime($list['iepg']) * 60;
			$elapsed = time() - getStartTime($list['iepg']);
			$pct	= $elapsed/$secs * 100 + 0.5;
			if (0 < $pct && $pct < 100)
				$percent = sprintf("<BR>(%d%%)", $pct);
		} else if ($dir && $dir == $avidir) {
			$pct = fsizeInKB("$avidir/$tsfile")/fsizeInKB("$tsdir/$tsfile")
				* 100 + 0.5;
			$percent = sprintf("<BR>(%d%%)", $pct);
		}
	}

	printf("<TD ALIGN='%s'>%s%s</TD>\n", $align, $stext, $percent);
}

//
// Show recorded program status
//
function showStatus($list) {
	global $thistab;

	$scolor	= array('Failed'	=> 'red',
					'Cancelled'	=> '#c0c0c0',
					'Aborted'	=> 'c0f0080',
					'Ready'		=> '#00d000',
					'Missed'	=> 'red',
					'Recording'	=> 'red',
					'Transcoding'	=> 'gray',
					'Decoding'	=> 'gray',
					'Waiting'	=> '#008080',
					'Xferring'	=> '#00d0d0',
					'Stopped'	=> 'red',
					'B25 decodeing'	=> 'gray',
					'B25 failed'	=> 'red',
					'FFMPEG failed'	=> 'red',
					);
	$action	= array('Failed'	=> 'delete',
					'Ready'		=> 'delete',
					'Aborted'	=> 'delete',
					'Missed'	=> 'delete',
					'Recording'	=> 'abort',
					'Transcoding'	=> 'abort',
					'Cancelled'	=> 'delete',
					'Xferring'	=> 'stop',
					'Complete'	=> 'delete',
					'Stopped'	=> 'delete'
					);
	$stext	= array('Recording'		=> 'Now<BR>Rec\'ing',
					'Cancelled'		=> 'Canc\'led',
					'Transcoding'	=> 'Trans-<BR>coding',
					);

	$stat	= readStat($list);
	$percent	= '';
	if ($stat == 'Ready') {
		@list($xfrd, $rate)	= split('@', readXfrd($list));
		if ($xfrd > 0) {
			$fname	= filenamepath($list, 'avi');
			$fsize	= fsizeInKB($fname);
			if ($fsize > 0) {
				$pct	= $xfrd/$fsize * 100 + 0.5;
				if ($pct > 99) {
					$pct = 99;
				}
				if (@$rate) {
					$percent = sprintf("<BR><FONT SIZE='-1'>%d%% ", $pct);
					if (filemtime(filenamepath($list, 'xfrd')) < time() - 120) {
						$percent .= "<B>STALLED</B>";
					} else if ($rate > 1000) {
						$percent .= sprintf("%.2fMB/s", $rate/1000);
					} else {
						$percent .= sprintf("%dkB/s", $rate);
					}
					$percent .= "</FONT>";
				} else
					$percent = sprintf("<BR>(%d%%)", $pct);
			}
			$stat	= 'Xferring';
		} else if ($xfrd < 0) {
			$stat	= 'Complete';
		}
	}

	$statt	= array_key_exists($stat, $stext) ? $stext[$stat] : $stat;

	$fcolor	= @$scolor[$stat];
	$sopen	= sprintf("<FONT %s SIZE='-1'><B>",
						$fcolor ? "COLOR='$fcolor'" : "");
	$sclose	= "</B></FONT>";
	if (@$action[$stat]) {
		$sopen	= sprintf("<A HREF='javascript:void(\"%s\");' " .
"onClick=\"window.open('6ga-panel.php?tab=%s&action=%s&file=%s&_time=%d'," .
		  "'_6ga_panel');\">%s",
			$list['fbasename'], $thistab,
			$action[$stat], $list['fbasename'], time(), $sopen);
		$sclose	.= "</A>";
	}
	printf("<TD ALIGN='center'>%s%s%s%s</TD>\n",
			$sopen, $statt, $sclose, $percent);
}

//
// Show "Cancel" column
//
function showCancel($list) {
	global $thistab;
	global $COLALIGN;

	printf("<TD ALIGN='%s'><A HREF='javascript:void(\"%s\");' " .
		"onClick=\"window.open('6ga.php?tab=%s&action=cancel&file=%s'," .
		"'_6ga');\"><B>Cancel</B></A></TD>\n",
		$COLALIGN['CANCEL'], $list['fbasename'], $thistab, $list['fbasename']);
}

//
// Show a program entry
//
function showList($list) {
	global $thistab;
	global $COLUMN, $COLSHOW;

	printf("<TR HEIGHT=32 BGCOLOR='%s'>\n", getListBGColor());
	foreach ($COLUMN[$thistab] as $col) {
		$COLSHOW[$col]($list);
	}
	printf("</TR>\n");
	return TRUE;
}

printDocHeader();
showGETArgs();

if ($thistab == 'recorded') {
	echo "<META HTTP-EQUIV='Refresh' CONTENT='60'>\n";
} else if ($thistab == 'scheduled') {
	echo "<META HTTP-EQUIV='Refresh' CONTENT='600'>\n";
}
echo <<<_EOS
<HTML>
<HEAD>
<STYLE TYPE="text/css">
<!--
A {text-decoration:none}
A.anchor {
	color: #000000;
}
A:visited {
	color: #0000ff;
}
A:hover {
	color: #ff3f00;
	font-weight: bold;
	text-decoration: underline;
}
-->
</STYLE>
<meta http-equiv="Content-Type" content="text/html;charset=shift_jis" />
<SCRIPT LANGUAGE='JavaScript'>
<!--
function refreshSView() {
	if ('$thistab' == 'scheduled') {
		for (var i = 0; i < window.parent.frames.length; i++) {
			var frame	= window.parent.frames[i];
			if (frame.name == '_6ga_main'
				&& (frame.location.pathname.match('6ga-sview.php') ||
					frame.location.pathname.match('/epgrec/index.php') ||
					frame.location.pathname.match('/epgrec/index2.php'))) {
				frame.location.reload();
				break;
			}
		}
	}
}
//-->
</SCRIPT>
</HEAD>
<BODY onLoad='refreshSView();'>
<TABLE WIDTH=100% BORDER=0 CELLSPACING=1 CELLPADDING=0>
_EOS;

showListHeader();

$nofRecords	= 0;
if (($list = openList($thistab)) != FALSE) {
	while (readList($list)) {
		if (showList($list))
			$nofRecords++;
	}
	closeList($list);
}

if ($nofRecords == 0) {
	printf("<TR><TD COLSPAN=%d><CENTER>No %s programs</CENTER></TD></TR>\n",
			count($COLUMN[$thistab]), $thistab);
}
?>
</TABLE>
</BODY>
</HTML>
